/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "DirItem.h"

DirItem::DirItem ( DirItem* parent )
{
	//qDebug() <<"creating DirItem";
	mparent = parent;
	fchild = NULL;
	mnext = NULL;
	properties<<""<<""<<QIcon();
	//qDebug() <<"created DirItem";
}

DirItem::DirItem ( const QList<QVariant> list, DirItem* parent )
{
	//qDebug() <<"creating DirItem";
	//if ( list.count() > 0 ) qDebug() <<"\twith data(0) = "<<list[0];
	properties = list;
	mparent = parent;
	fchild = NULL;
	mnext = NULL;
	//qDebug() <<"created DirItem";
}

DirItem::~DirItem()
{
	if ( this != NULL )
	{
		//qDebug() <<"DirItem::destructor: deleting "<<dataAt ( 0 ).toString();//<<myRow() <<std::endl;
		if ( fchild != NULL )
		{
			DirItem* iter = fchild;
			while ( iter->mnext != NULL )
			{
				DirItem* tmp = iter;
				iter = iter->mnext;
				delete tmp;
				tmp = NULL;
			}
		}
		//qDebug() <<"deleted";
		//std::cout<<"deleted"<<std::endl;
	}
	/*else
	{
		std::cout<<"DirItem::destructor: already null!"<<std::endl;
	}*/
}


int DirItem::columns()
{
	return properties.count();
}

int DirItem::myRow()
{
	//qDebug() <<"DirItem::myRow(): my data is: "<<dataAt(0).toString();
	if ( mparent == NULL )
	{
		//qDebug() <<"begin DirItem::myRow(): mparent == NULL -> I am the first!!";
		return 0; //I am the first! (rootItem)
	}
	//qDebug() <<"DirItem::myRow(): mparent != NULL";
	//qDebug() <<"DirItem::myRow(): mparent data "<<mparent->dataAt(0) ;
	int i=0;
	DirItem* tmp = mparent->fchild;
	if ( tmp == NULL ) //should never happen
	{
		qDebug() <<"DirItem::myRow(): invalid tree - parent has children, but first is NULL";
		return -1;
	}
	//qDebug() <<"DirItem::myRow(): mparent->fchild != NULL";
	while ( tmp != this )
	{
		//qDebug() <<"DirItem::myRow(): item at "<<i<<" is not me!";
		//qDebug() <<"DirItem::myRow(): item at "<<i<<" has data "<<tmp->dataAt(0);
		i++;
		if ( tmp->mnext == NULL )
		{
			qDebug() <<"DirItem::myRow(): invalid tree - I do not belong to my parent";
			return -1;
		}
		tmp = tmp->mnext;
		if ( tmp == NULL ) //paranioc check
		{
			qDebug() <<"DirItem::myRow(): BUG!!!! tmp == NULL";
			return -1;
		}
	}
	//qDebug() <<"DirItem::myRow(): now returning "<<i;
	return i;
}

QVariant DirItem::dataAt ( int column )
{
	//qDebug() <<"begin DirItem::dataAt ( "<<column<<" )";
	if ( column >= properties.count() )
	{
		qDebug() <<"DirItem::dataAt: column "<<column<<": index out of range";
		return QVariant();
	}
	//qDebug() <<"DirItem::dataAt(): now returning";
	return properties.at ( column );

}

void DirItem::setData ( QVariant data, int column )
{
	//qDebug() <<"begin DirItem::setData() for column "<<column;
	if ( column >= properties.count() )
	{
		qDebug() <<"DirItem::setData(): column "<<column<<": index out of range";
		return;
	}
	properties[column] = data;
	//qDebug() <<"end DirItem::setData() for column "<<column;
}

void DirItem::setAllData ( const QList<QVariant> data )
{
	//qDebug() <<"begin DirItem::setAllData()";
	properties = data;
	//qDebug() <<"end DirItem::setAllData()";
}

void DirItem::setDataforAll ( QVariant data, int column )
{
	//qDebug() <<"DirItem::setDataforAll";
	setData ( data, column );
	if ( fchild == NULL )
	{
		//qDebug() <<"DirItem::setDataforAll: fchild == NULL";
		return;
	}
	DirItem* tmp = fchild;
	tmp->setDataforAll ( data, column );
	while ( tmp->mnext != NULL )
	{
		tmp = tmp->mnext;
		tmp->setDataforAll ( data, column );
	}
}
int DirItem::children()
{
	//qDebug() <<"begin DirItem::children()";
	if ( fchild == NULL )
	{
		//qDebug() <<"DirItem::children(): no children";
		return 0;
	}
	int i=1;
	DirItem* tmp = fchild;
	while ( tmp->mnext != NULL )
	{
		i++;
		tmp = tmp->mnext;
	}
	//qDebug() <<dataAt(0)<<" : has "<<i<<" children\n\t->now returning";
	return i;
}

DirItem* DirItem::childAt ( int row )
{
	//qDebug() <<"begin DirItem::childAt ( "<<row<<" )";
	if ( fchild == NULL )
	{
		//qDebug() <<"DirItem::childAt: no children";
		return NULL;
	}
	if ( row == 0 ) return fchild;
	int i=0;
	DirItem* tmp = fchild;
	while ( i<row-1 && tmp->mnext != NULL )
	{
		i++;
		tmp = tmp->mnext;
	}
	if ( i != row-1 )
	{
		//qDebug() <<"begin DirItem::childAt: i != row-1 -> got NULL";
		return NULL;
	}
	else if ( tmp->mnext == NULL )
	{
		//qDebug() <<"begin DirItem::childAt: is NULL";
		return NULL;
	}
	return tmp->mnext;
}
DirItem* DirItem::lastChild()
{
	if ( fchild == NULL ) return NULL;
	return childAt ( children()-1 );
}
void DirItem::createFChild ( const QList<QVariant> list )
{
	//qDebug() <<"begin DirItem::createFChild";
	if ( fchild == NULL )
	{
		fchild = new DirItem ( list, this );
	}
	else
	{
		if ( mnext == NULL )
		{
			mnext = fchild;
			fchild = new DirItem ( list, this );
		}
		else
		{
			DirItem* tmp = mnext;
			mnext = fchild;
			fchild = new DirItem ( list, this );
			fchild->mnext = tmp;
		}
	}
	//qDebug() <<"end DirItem::createFChild";
}

void DirItem::createNextChild ( const QList<QVariant> list )
{
	//qDebug() <<"begin DirItem::createNextChild";
	if ( mnext == NULL )
	{
		if ( mparent == NULL )	mnext = new DirItem ( list );
		else	mnext = new DirItem ( list, mparent );
	}
	else
	{
		DirItem* tmp = mnext;
		if ( mparent == NULL )	mnext = new DirItem ( list );
		else	mnext = new DirItem ( list, mparent );
		mnext->mnext = tmp;
	}
	//qDebug() <<"end DirItem::createNextChild";
}
void DirItem::insertChild ( DirItem* child, int row )
{
	if ( fchild == NULL )
	{
		fchild = child;
		fchild->mparent = this;
		fchild->mnext = NULL;
		return;
	}
	if ( row == 0 )
	{
		DirItem* tmp = fchild;
		fchild = child;
		fchild->mparent = this;
		fchild->mnext = tmp;
	}
	else if ( row < children() )
	{
		DirItem* tmp = childAt ( row );
		childAt ( row-1 )->mnext = child;
		child->mparent = this;
		child->mnext = tmp;
	}
	else
	{
		lastChild()->mnext = child;
		child->mparent = this;
		child->mnext = NULL;
	}
}
bool DirItem::appendChild ( const QList<QVariant> list )
{
	if ( children() == 0 ) createFChild ( list );
	else
	{
		DirItem* tmp = childAt ( children()-1 );
		if ( tmp == NULL ) return false;
		tmp->createNextChild ( list );
	}
	return true;
}
bool DirItem::transferChildren()
{
	if ( mparent == NULL )
	{
		//qDebug() <<"DirItem::transferChildren(): mparent = NULL";
		return false;
	}
	int r = myRow();

	if ( fchild == NULL )
	{
		/*qDebug() <<"DirItem::transferChildren(): no children";
		if ( mnext != NULL )
		{
			if ( r == 0 )
			{
				mparent->fchild = mnext;
			}
			else
			{
				mparent->childAt ( r-1 )->mnext = mnext;
			}
		}
		else
		{
			if ( r == 0 )
			{
				mparent->fchild = NULL;
			}
			else
			{
				mparent->childAt ( r-1 )->mnext = NULL;
			}
		}*/
		return true;
	}

	DirItem* tmp = fchild;
	mparent->insertChild ( tmp, r+1 );
	r++;
	while ( tmp->mnext != NULL )
	{
		tmp = tmp->mnext;
		mparent->insertChild ( tmp, r+1 );
		r++;
	}
	reset();
	return true;
}
void DirItem::reset()
{
	mnext = NULL;
	fchild = NULL;
}
void DirItem::removeFChild()
{
	//qDebug() <<"begin DirItem::removeFChild()";
	if ( fchild == NULL ) return; //already no child
	DirItem* tmp = fchild;
	if ( fchild->mnext != NULL )
	{
		fchild = fchild->mnext;
	}
	else fchild = NULL;
	delete tmp;
	tmp = NULL;
	//qDebug() <<"end DirItem::removeFChild()";
}

void DirItem::removeNextChild()
{
	//qDebug() <<"begin DirItem::removeNextChild()";
	if ( mnext == NULL ) return; //already removed
	if ( mnext->mnext != NULL )
	{
		DirItem* tmp = mnext->mnext;
		delete mnext;
		mnext = tmp;
	}
	else
	{
		delete mnext;
		mnext = NULL;
	}
	//qDebug() <<"end DirItem::removeNextChild()";
}

int DirItem::rowOfChild ( DirItem* child )
{
	//qDebug() <<"begin DirItem::rowOfChild";
	if ( fchild == NULL )
	{
		//qDebug() <<"DirItem::rowOfChild: no children";
		return -1;
	}
	DirItem* tmp = fchild;
	bool found = true;
	int i=0;
	while ( tmp != child )
	{
		if ( tmp->mnext == NULL )
		{
			found = false;
			return -1;
		}
		tmp = tmp->mnext;
		i++;
	}
	return i;
}

QStringList DirItem::getDirData ( bool include_my_data )
{
	QStringList list;
	if ( include_my_data )
	{
		if ( !dataAt ( 0 ).toString().isEmpty() && dataAt ( 1 ).toBool() )	list<<dataAt ( 0 ).toString();
	}
	if ( fchild == NULL ) return list;
	DirItem* tmp = fchild;
	list<<tmp->getDirData ( true );
	while ( tmp->mnext != NULL )
	{
		tmp = tmp->mnext;
		list<<tmp->getDirData ( true );
	}
	return list;
}

void DirItem::print()
{
	std::cout<<dataAt ( 0 ).toString().toStdString().c_str() <<std::endl;
	if ( fchild == NULL ) return;
	std::cout<<"\t";
	DirItem* tmp = fchild;
	tmp->print();
	while ( tmp->mnext != NULL )
	{
		tmp = tmp->mnext;
		std::cout<<"\t";
		tmp->print();
	}
}
