/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "DirModel.h"

#include <iostream>
DirModel::DirModel ( QObject *parent ) : QAbstractItemModel ( parent )
{
	//qDebug() <<"creating DirModel";
	//std::cout<<"creating DirModel"<<std::endl;
	QList<QVariant> data;
	data << tr ( "Directory" ) << tr ( "Enabled" ) <<tr ( "Base directory" );
	root = new DirItem ( data );
	baseItem = NULL;
	//qDebug() <<"created DirModel";
}

DirModel::~DirModel()
{
	if ( this != NULL )
	{
		//std::cout<<"begin ~DirModel()"<<std::endl;
		delete root;
		root = NULL;
		//std::cout<<"end ~DirModel()"<<std::endl;
	}
	else
	{
		//std::cout<<"DirModel::destructor: already null!"<<std::endl;
	}
}

QModelIndex DirModel::index ( int row, int column, const QModelIndex &index ) const
{
	//qDebug() <<"begin  DirModel::index";
	if ( !hasIndex ( row, column, index ) )	return QModelIndex();

	DirItem *tmp;
	if ( !index.isValid() )	tmp = root;
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );
	DirItem *child = tmp->childAt ( row );
	if ( child == NULL ) return QModelIndex();
	return createIndex ( row, column, child );
}

QModelIndex DirModel::parent ( const QModelIndex &index ) const
{
	//qDebug() <<"begin DirModel::parent";
	if ( index.isValid() )
	{
		DirItem *child = static_cast<DirItem*> ( index.internalPointer() );
		if ( child->mparent == root )	return QModelIndex();
		return createIndex ( child->mparent->myRow(), 0, child->mparent );
	}
	return QModelIndex();
}

int DirModel::rowCount ( const QModelIndex &index ) const
{
	//qDebug() <<"begin DirModel::rowCount";

	if ( index.column() > 0 )
	{
		//qDebug() <<"DirModel::rowCount(): index.column() > 0 -> returning 0";
		return 0;
	}

	DirItem* tmp;

	if ( !index.isValid() )
	{
		tmp = root;
		//qDebug() <<"DirModel::rowCount(): tmp is root!";
	}
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );

	//qDebug() <<"DirModel::rowCount(): paranioc check of pointer";
	if ( tmp == NULL ) //paranoic check
	{
		//qDebug() <<"DirModel::rowCount(): tmp == NULL -> returning 0";
		return 0;
	}

	//qDebug() <<"DirModel::rowCount(): tmp != NULL ->returning tmp->children() = "<<tmp->children();
	//qDebug() <<"DirModel::rowCount(): now returning";
	return tmp->children();
}

bool DirModel::hasChildren ( const QModelIndex & index ) const
{
	//qDebug() <<"begin DirModel::hasChildren";
	if ( index.isValid() )	return static_cast<DirItem*> ( index.internalPointer() )->fchild == NULL ? false : true;
	return root->fchild == NULL ? false : true;
}

int DirModel::columnCount ( const QModelIndex &index ) const
{
	//qDebug() <<"begin DirModel::columnCount";
	if ( index.isValid() )	return static_cast<DirItem*> ( index.internalPointer() )->columns();
	return root->columns();
}

QVariant DirModel::data ( const QModelIndex &index, int role ) const
{
	//qDebug() <<"begin DirModel::data";
	if ( !index.isValid() )
	{
		//qDebug() <<"DirModel::data(): invalid index -> returing empty data";
		return QVariant();
	}

	DirItem *tmp = static_cast<DirItem*> ( index.internalPointer() );

	if ( tmp == NULL )
	{
		//qDebug() <<"DirModel::data(): static_cast returned NULL -> returning empty data";
		return QVariant();
	}
	//qDebug() <<"DirModel::data(): tmp != NULL -> returning->";
	//qDebug() <<"-> tmp->dataAt("<<index.column() <<")";

	if ( role == Qt::FontRole && index.column() == 0 )
	{
		QFont font;
		font.setItalic ( !tmp->dataAt ( 1 ).toBool() );
		return font;
	}
	if ( role == Qt::DisplayRole && index.column() == 0 )
		return tmp->dataAt ( 0 );
	else if ( role == Qt::CheckStateRole && index.column() == 1 )
		return tmp->dataAt ( 1 ) == true ? Qt::Checked : Qt::Unchecked ;
	else if ( role == Qt::DecorationRole && index.column() == 2 )
		return tmp->dataAt ( 2 ) == true ? QIcon ( ":/emblem-important.svg" ) : QIcon ( ":/emblem-important-disabled.svg" );
	return QVariant();
}

Qt::ItemFlags DirModel::flags ( const QModelIndex &index ) const
{
	//qDebug() <<"begin DirModel::flags";
	if ( !index.isValid() )	return Qt::NoItemFlags;
	else
	{
		if ( index.column() == 1 )	return ( Qt::ItemIsEnabled | Qt::ItemIsSelectable|Qt::ItemIsUserCheckable );
		return ( Qt::ItemIsEnabled | Qt::ItemIsSelectable );
	}
}

QVariant DirModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
	//qDebug() <<"begin DirModel::headerData ";
	if ( orientation == Qt::Horizontal && ( role == Qt::DisplayRole || role == Qt::DecorationRole ) )
		return root->dataAt ( section );
	return QVariant();
}

/*bool DirModel::insertRow ( int pos, const QModelIndex &index )
{
	//qDebug() <<"begin DirModel::insertRow";
	DirItem* tmp;
	if ( !index.isValid() ) tmp = root;
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );

	if ( tmp == NULL ) return false; //paranoid check :)

	if ( ( pos < 0 ) || ( pos > tmp->children() ) )
	{
		qDebug() <<"DirModel::insertRow: cannot insert: index out of range";
		return false;
	}

	beginInsertRows ( index, pos, pos );
	if ( pos == 0 )
	{
		tmp->createFChild();
	}
	else
	{
		tmp = tmp->childAt ( pos-1 );
		if ( tmp == NULL )
		{
			qDebug() <<"DirModel::insertRow: got NULL from childAt("<<pos<<")";
			endInsertRows();
			return false;
		}
		tmp->createNextChild();
	}
	endInsertRows();

	return true;
}

bool DirModel::insertRows ( int pos, int rows, const QModelIndex &index )
{
	//qDebug() <<"begin DirModel::insertRows";
	DirItem* tmp;
	if ( !index.isValid() ) tmp = root;
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );

	if ( tmp == NULL ) return false; //paranoid check :)

	if ( ( pos < 0 ) || ( pos > tmp->children() ) )
	{
		qDebug() <<"cannot insert: index out of range";
		return false;
	}

	beginInsertRows ( index, pos, pos + rows - 1 );
	if ( pos == 0 )
	{
		tmp->createFChild();
		for ( int i=0;i<rows-1;i++ )
		{
			tmp->createNextChild();
			tmp = tmp->mnext;
		}
	}
	else
	{
		tmp = tmp->childAt ( pos-1 );
		if ( tmp == NULL )
		{
			qDebug() <<"DirModel::insertRows: got NULL from childAt("<<pos<<")";
			endInsertRows();
			return false;
		}
		for ( int i=0;i<rows;i++ )
		{
			tmp->createNextChild();
			tmp = tmp->mnext;
		}
	}
	endInsertRows();

	return true;
}

bool DirModel::removeRow ( int pos, const QModelIndex &index )
{
	//qDebug() <<"begin DirModel::removeRow";
	DirItem* tmp;
	if ( !index.isValid() ) tmp = root;
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );

	if ( tmp == NULL ) return false; //paranoid check :)

	if ( ( pos<0 ) || ( pos >= tmp->children() ) )
	{
		qDebug() <<"cannot insert: index out of range";
		return false;
	}

	beginRemoveRows ( index, pos, pos );
	if ( pos == 0 )
	{
		tmp->removeFChild();
	}
	else
	{
		tmp = tmp->childAt ( pos-1 );
		if ( tmp == NULL )
		{
			qDebug() <<"DirModel::removeRow: got NULL from childAt("<<pos-1<<")";
			endRemoveRows();
			return false;
		}
		tmp->removeNextChild();
	}
	endRemoveRows();

	return true;
}

bool DirModel::removeRows ( int pos, int rows, const QModelIndex &index )
{
	//qDebug() <<"begin DirModel::removeRows";
	DirItem* tmp;
	if ( !index.isValid() ) tmp = root;
	else	tmp = static_cast<DirItem*> ( index.internalPointer() );

	if ( tmp == NULL ) return false; //paranoid check :)

	if ( ( pos<0 ) || ( pos+rows > tmp->children() ) )
	{
		qDebug() <<"cannot insert: index out of range";
		return false;
	}

	beginRemoveRows ( index, pos, pos + rows - 1 );
	if ( pos == 0 )
	{
		for ( int i=0;i<rows;i++ )	tmp->removeFChild();
	}
	else
	{
		tmp = tmp->childAt ( pos-1 );
		if ( tmp == NULL )
		{
			qDebug() <<"DirModel::removeRows: got NULL from childAt("<<pos-1<<")";
			endRemoveRows();
			return false;
		}
		for ( int i=0;i<rows;i++ )	tmp->removeNextChild();
	}
	endRemoveRows();

	return true;
}*/

int DirModel::childrenAtIndex ( QModelIndex index )
{
	if ( !index.isValid() )
	{
		return root->children();
	}
	DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
	if ( tmp == NULL )
	{
		//qDebug() <<"DirModel::removeAt(): NULL";
		return 0;
	}
	return tmp->children();

}

bool DirModel::containsBaseItem ( QModelIndex index )
{
	if ( !index.isValid() )
	{
		//qDebug() <<"DirModel::removeAt(): root contains all!";
		return true;
	}
	DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
	if ( tmp == NULL )
	{
		//qDebug() <<"DirModel::removeAt(): NULL item";
		return true; //really, it's a BUG!
	}
	if ( baseItem == NULL ) return false;
	if ( tmp == baseItem ) return true;
	DirItem* baseP;
	while ( baseItem->mparent != NULL )
	{
		baseP = baseItem->mparent;
		if ( baseP == tmp ) return true;
		if ( baseP == root ) return false;
	}
	return false;
}

void DirModel::removeAt ( QModelIndex index )
{
	if ( !index.isValid() )
	{
		//qDebug() <<"DirModel::removeAt(): cannot delete root!";
		return;
	}
	DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
	if ( tmp == NULL || tmp->mparent == NULL )
	{
		//qDebug() <<"DirModel::removeAt(): NULL item or NULL parent";
		return;
	}
	DirItem* tmpP = tmp->mparent;
	QModelIndex indexP = ( tmpP == root ) ? QModelIndex() : createIndex ( tmpP->myRow(), 0, tmpP );
	int pos = tmp->myRow();

	beginRemoveRows ( indexP, pos, pos );
	if ( pos == 0 ) tmpP->removeFChild();
	else tmpP->childAt ( pos-1 )->removeNextChild();
	endRemoveRows();
}

DirItem* DirModel::getRoot()
{
	//qDebug() <<"begin DirModel::getRoot()";
	return root;
}

DirItem* DirModel::getBase()
{
	//qDebug() <<"begin DirModel::getRoot()";
	return baseItem;
}

bool DirModel::setFirstAsBase()
{
	if ( root->fchild == NULL ) return false;
	DirItem* tmp = root->fchild;
	bool base = tmp->dataAt ( 2 ).toBool();
	//qDebug() <<"tmp->dataAt ( 2 ).toBool() = "<<base;
	if ( base )	return true;
	if ( baseItem != NULL )
	{
		//qDebug() <<"baseItem->data = "<<baseItem->dataAt ( 0 );
		baseItem->setData ( false, 2 );
		QModelIndex baseIndex = ( baseItem == root ) ? QModelIndex() : createIndex ( baseItem->myRow(), 2, baseItem );
		emit dataChanged ( baseIndex, baseIndex );
	}
	tmp->setData ( true, 2 );
	baseItem = tmp;
	QModelIndex baseIndex = createIndex ( 0, 2, baseItem );
	emit dataChanged ( baseIndex, baseIndex );

	return true;
}

void DirModel::setup ( QStringList allDirs )
{
	DirItem* tmp = root;

	beginInsertRows ( QModelIndex(), root->children(), root->children() );
	if ( baseItem != NULL ) tmp->appendChild ( QList<QVariant>() <<allDirs[0]<<true<<false );
	else
	{
		tmp->appendChild ( QList<QVariant>() <<allDirs[0]<<true<<true );
	}
	emit dir_added ( 1 );
	endInsertRows();

	if ( tmp->lastChild() == NULL )
	{
		qDebug() <<"DirMode::setup(): failed to insert 1st row";
		return;
	}
	tmp = tmp->lastChild();
	if ( baseItem == NULL )	baseItem = tmp;
	for ( int i=1;i<allDirs.count();i++ )
	{
		if ( allDirs[i].contains ( allDirs[i-1] ) ) //subdir
		{
			beginInsertRows ( createIndex ( tmp->myRow(), 0, tmp ), 0, 0 );
			tmp->createFChild ( QList<QVariant>() <<allDirs[i]<<true<<false );
			emit dir_added ( i+1 );
			endInsertRows();

			if ( tmp->fchild == NULL )
			{
				//qDebug() <<"DirMode::setup(): error: fchild = NULL";
				return;
			}
			tmp = tmp->fchild;
		}
		else
		{
			QString dir;
			do
			{
				if ( tmp->mparent == NULL )
				{
					//qDebug() <<"DirMode::setup(): not subdir of first!";
					tmp = root;
					break;
				}
				tmp = tmp->mparent;
				dir = tmp->dataAt ( 0 ).toString();
			}
			while ( !allDirs[i].contains ( dir ) );

			beginInsertRows ( createIndex ( tmp->myRow(), 0, tmp ), 0, 0 );
			tmp->appendChild ( QList<QVariant>() <<allDirs[i]<<true<<false );
			emit dir_added ( i+1 );
			endInsertRows();

			tmp = tmp->lastChild();
			if ( tmp == NULL )
			{
				//qDebug() <<"DirMode::setup(): error: lastChild = NULL";
				return;
			}
		}
	}
}
void DirModel::changeDataAtIndex ( QModelIndex index, QVariant data )
{
	DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
	if ( tmp == NULL )
	{
		//qDebug() <<"clicked NULL item?";
		return;
	}
	tmp->setData ( data, index.column() );
	emit dataChanged ( index, index );
}
void DirModel::switchDataAtIndex ( QModelIndex index, bool subdirs )
{
	int col = index.column();
	//qDebug() <<"switching data at "<<col;
	if ( col < 1 )
	{
		//qDebug() <<"data at this column is not of type bool";
		return;
	}
	DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
	if ( tmp == NULL )
	{
		//qDebug() <<"clicked NULL item?";
		return;
	}
	if ( col == 1 )
	{
		bool data = tmp->dataAt ( col ).toBool();
		if ( subdirs )
		{
			DirItem* tmpC = tmp;
			while ( tmpC->lastChild() != NULL ) tmpC = tmpC->lastChild();
			QModelIndex indexC = ( tmpC == root ) ? QModelIndex() : createIndex ( tmpC->myRow(), col, tmpC );
			tmp->setDataforAll ( !data , col );
			emit dataChanged ( index, indexC );
			indexC = ( tmpC == root ) ? QModelIndex() : createIndex ( tmpC->myRow(), 0, tmpC );
			emit dataChanged ( index, indexC );
		}
		else
		{
			tmp->setData ( !data, col );
			emit dataChanged ( index, index );
		}
	}
	else if ( col == 2 )
	{
		bool base = tmp->dataAt ( col ).toBool();
		//qDebug() <<"tmp->dataAt ( 2 ).toBool() = "<<base;
		if ( !base )
		{
			if ( baseItem != NULL )
			{
				//qDebug() <<"baseItem->data = "<<baseItem->dataAt ( 0 );
				baseItem->setData ( false, col );
				QModelIndex baseIndex = ( baseItem == root ) ? QModelIndex() : createIndex ( baseItem->myRow(), col, baseItem );
				emit dataChanged ( baseIndex, baseIndex );
			}
			tmp->setData ( true, col );
			baseItem = tmp;
			emit dataChanged ( index, index );
		}
		else
		{
			if ( baseItem != NULL )
			{
				//qDebug() <<"baseItem->data = "<<baseItem->dataAt ( 0 );
				baseItem->setData ( false, col );
				QModelIndex baseIndex = ( baseItem == root ) ? QModelIndex() : createIndex ( baseItem->myRow(), col, baseItem );
				emit dataChanged ( baseIndex, baseIndex );
				baseItem = NULL;
			}
		}
	}
}
/*////////OLD SETUP//////////////////////////////////////////////
		if(allDirs[i].contains(allDirs[i-1])) //subdir
		{
			qDebug()<<"DirMode::setup(): doing down";
			qDebug()<<"as "<<allDirs[i]<<" is subdirectory of "<<allDirs[i-1];
			///////paranoidal check/////////
			tmp = static_cast<DirItem*>(index.internalPointer());
			if(tmp == NULL)	qDebug()<<"DirMode::setup(): parent index points to NULL item";
			else	qDebug()<<"DirMode::setup(): parent index points to "<<tmp->dataAt(0).toString();
			////////////////////////////////
			if(!model->insertRow(0, index))
			{
				qDebug()<<"DirMode::setup(): failed to insert row - i="<<i;
				return;
			}
			index = model->index(0, 0, index);
			tmp = static_cast<DirItem*>(index.internalPointer());
			if(tmp == NULL)
			{
				qDebug()<<"DirMode::setup(): cannot set data (1): tmp == NULL";
			}
			else
			{
				qDebug()<<"DirMode::setup(): checking data before setting:\n(should be empty) - data:"<<tmp->dataAt(0).toString();
				tmp->setAllData(QList<QVariant>()<<allDirs[i]<<"true"<<QIcon(":/emblem-important-disabled.svg"));
			}
		}
		else
		{
			qDebug()<<"DirMode::setup(): going up";
			int k=i-2;
			qDebug()<<"DirMode::setup(): k="<<k;
			while(k >= 0 && !allDirs[i].contains(allDirs[k]))
			{
				QModelIndex tmpInd = model->parent(index);
				if(!tmpInd.isValid())
				{
					qDebug()<<"DirMode::setup(): tmpInd is invalid";
					return;
				}
				tmp = static_cast<DirItem*>(tmpInd.internalPointer());
				if(tmp == NULL)
				{
					qDebug()<<"DirMode::setup(): cannot get parent: tmp == NULL";
					return;
				}
				int oldk = k;
				k = allDirs.indexOf(tmp->dataAt(0).toString());
				if(k < 0 || k>= oldk  )
				{
					qDebug()<<"DirMode::setup(): indexOf returned -1 or smth greater than used to be - BUG!!!";
					return;
				}
				qDebug()<<"DirMode::setup(): new k="<<k;
				index = model->parent(index);
			}
			int mrow = index.row();
			///////paranoidal check/////////
			tmp = static_cast<DirItem*>(index.internalPointer());
			if(tmp == NULL)	qDebug()<<"DirMode::setup(): index points to NULL item";
			else	qDebug()<<"DirMode::setup(): index points to "<<tmp->dataAt(0).toString();
			////////////////////////////////
			index = model->parent(index);
			qDebug()<<"DirMode::setup(): "<<allDirs[i]<<" is subdirectory of "<<allDirs[k];
			///////paranoidal check/////////
			tmp = static_cast<DirItem*>(index.internalPointer());
			if(tmp == NULL)	qDebug()<<"DirMode::setup(): parent index points to NULL item";
			else	qDebug()<<"DirMode::setup(): parent index points to "<<tmp->dataAt(0).toString();
			qDebug()<<"DirMode::setup(): row= "<<mrow;
			////////////////////////////////

			if(!model->insertRow(mrow+1, index))
			{
				qDebug()<<"DirMode::setup(): failed to insert row - i="<<i;
				return;
			}
			index = model->index(mrow+1, 0, index);
			tmp = static_cast<DirItem*>(index.internalPointer());
			if(tmp == NULL)
			{
				qDebug()<<"DirMode::setup(): cannot set data (2): tmp == NULL";
			}
			else
			{
				tmp->setAllData(QList<QVariant>()<<allDirs[i]<<"true"<<QIcon(":/emblem-important-disabled.svg"));
			}
		}
////////////////////////////////////////////////////////////////*/
