/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>
#include "qrcgenerator.h"
#include "qrcgenerator_gui.h"
#include "qrcgenerator_generator.h"
#include "qrcgenerator_save.h"
#include "qrcgenerator_cmd.h"

QRCGenerator::QRCGenerator()
{
	imgformats.clear();
	imgformat = tr ( "Images" ) +" (";
	foreach ( QByteArray ba, QImageReader::supportedImageFormats() )
	{
		imgformats<<"*."+QString ( ba );
		imgformat+=" *."+QString ( ba );
	}
	imgformat+=")";
}

void QRCGenerator::on_aboutButton_clicked()
{
	QMessageBox::about ( this, tr ( "About QRCGenerator %1" ).arg ( version ),tr ( "QRCGenerator creates .qrc files for Qt projects. For more information about .qrc files, see Qt Manual.<br>Brief help:<br>QRCGenerator can work in two modes: <b>gui</b> mode and <b>console</b> mode<br>\t(with and without graphical interface respectively )<br>To get information about <b>console</b> mode, run: \"qrcgenerator -h\" or \"qrcgenerator --help\"<br><b>gui</b> mode is switched on automatically if the program is started with no command line arguments.<br><br><b>Directories list:</b><br><b>Base</b> directory - the one, that contains *.pro file<br><b>Enabled</b> column - when you press <b>Generate</b> button, only checked directories from the list are scanned.<br><br><b>Formats list</b>:<br>Only files with selected extensions are added to the output file.<br><br><b>Output:</b><br>When you press <b>Generate</b> button, the output is written (or appended) to text edit. So you can preview and edit the output before it is written to output file. To save the output, press <b>Save</b> button (or <b>Copy</b> button to copy the output to clipboard). You will be asked for filename (the file will be placed in <b>Base</b> directory). If the file with specified name exists, you can overwrite it or append to it (<b>Warning!</b> it is assumed, that exiting file is valid, i.e. ends with \"&lt;/RCC&gt;\" - case insensitive).<br><br>To add current alias or prefix to corresponding drop-down list, press Return after you have finished editing it.<br><br>If you double click on a directory from the list, a list of files (filtered according to <b>Formats list</b>) will be shown instead of text edit for 10 seconds<br><br>QRCGenerator uses icons from <a href=\"http://tango.freedesktop.org/Tango_Desktop_Project\">Tango Desktop Project</a>" ) );
}

void QRCGenerator::on_addFormat_clicked()
{
	QString aform = QInputDialog::getText ( this, tr ( "QRCGenerator" ), tr ( "Extension: " ) );
	if ( aform.isEmpty() ) return;
	if ( !aform.startsWith ( "*." ) ) aform = "*."+aform;
	aform = aform.trimmed();
	if ( formats.contains ( aform ) )
	{
		QMessageBox::information ( this, tr ( "QRCGenerator" ), tr ( "Already in list" ) );
		return;
	}
	formats<<aform;
	formatsList->addItem ( aform );
}

void QRCGenerator::on_addImgFormats_clicked()
{
	QStringList items = formats<<imgformats;
	items.removeDuplicates();
	formats = items;
	formatsList->clear();
	formatsList->addItems ( formats );
}

void QRCGenerator::on_removeFormats_clicked()
{
	QList<QListWidgetItem*> items = formatsList->selectedItems();
	foreach ( QListWidgetItem* citem, items )
	{
		int r = formatsList->row ( citem );
		if ( r>=0 )
		{
			formats.removeAll ( citem->text() );
			delete formatsList->takeItem ( r );
		}
	}
}

void QRCGenerator::on_clearFormats_clicked()
{
	formatsList->clear();
	formats.clear();
}


void QRCGenerator::on_closeButton_clicked()
{
	//writeSettings();
	close();
}

void QRCGenerator::on_copyButton_clicked()
{
	if ( !plainTextEdit->toPlainText().isEmpty() )	clip->setText ( plainTextEdit->toPlainText() );
}
void QRCGenerator::on_addButton_clicked()
{
	QString dir = QFileDialog::getExistingDirectory ( this, tr ( "Select directory" ), lastdir );
	if ( dir.isEmpty() ) return;
	QDir ldir ( dir );
	ldir.cdUp();
	lastdir = ldir.canonicalPath();
	QStringList allDirs;

	if ( QMessageBox::question ( this, tr ( "QRCGenerator" ), tr ( "Add subdirectories?" ), QMessageBox::No|QMessageBox::Yes ) == QMessageBox::Yes )
	{
		QDirIterator iter ( dir, QDir::NoDotAndDotDot|QDir::Dirs, QDirIterator::Subdirectories );
		while ( iter.hasNext() ) allDirs<<iter.next();
	}

	allDirs.insert ( 0, dir );
	progBar->setMaximum ( allDirs.count() );
	progBar->setValue ( 0 );
	progBar->show();
	//qDebug() <<"QRCGenerator::on_addButton_clicked(): tree creation started!\nDirectories:\n"<<allDirs.join ( "\n" );
	model->setup ( allDirs );
	treeView->expandAll();
	for ( int col = 0; col < model->columnCount(); col++ )	treeView->resizeColumnToContents ( col );
	progBar->hide();

}

void QRCGenerator::on_clearButton_clicked()
{
	if ( plainTextEdit->toPlainText().isEmpty() ) return;
	if ( QMessageBox::question ( this, tr ( "QRCGenerator" ), tr ( "Clear text edit?" ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::Yes ) plainTextEdit->clear();
}
/*void QRCGenerator::on_treeWidget_itemClicked ( QTreeWidgetItem * item, int column )
{
	Qt::CheckState ch = item->checkState ( column );
	bool checked = ch == Qt::Checked ? true : false;
	foreach ( QTreeWidgetItem* item1, treeWidget->findItems ( item->text ( column ), Qt::MatchContains, column ) )
	{
		item1->setCheckState ( column, ch );
	}
}*/

/*void QRCGenerator::view_entered ( QModelIndex index )
{
	if ( index.column() == 0 )
	{
		DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
		if ( tmp == NULL )
		{
			qDebug() <<"hovered NULL item?";
			return;
		}
		QString dir = tmp->dataAt ( 0 ).toString();
		QStringList files = QDir(dir).entryList(formats, QDir::Files, QDir::Name );
		QString tooltip
	}
}*/

void QRCGenerator::hideBrower_timeout()
{
	//qDebug() <<"timeout";
	textBrowser->hide();
	plainTextEdit->show();
	hideBrowser->stop();
}

void QRCGenerator::view_doubleClicked ( QModelIndex index )
{
	switch ( index.column() )
	{
		case 0:
		{
			DirItem* tmp = static_cast<DirItem*> ( index.internalPointer() );
			if ( tmp == NULL )
			{
				//qDebug() <<"clicked NULL item?";
				return;
			}
			QString dir = tmp->dataAt ( 0 ).toString();
			QStringList files = QDir ( dir ).entryList ( formats, QDir::Files, QDir::Name );
			textBrowser->setHtml ( "<html><head></head><body><ul><uh>" + tr ( "Files in <b>%1</b>" ).arg ( dir ) + "</uh><li>" + files.join ( "<li>" ) + "</ul></body></html>" );
			plainTextEdit->hide();
			textBrowser->show();
			if ( hideBrowser->isActive() ) hideBrowser->stop();
			hideBrowser->start ( 10000 );
			break;
		}
		default: break;
	}
}
void QRCGenerator::view_clicked ( QModelIndex index )
{
	if ( !index.isValid() )	return;
	//qDebug() <<"clicked at column "<<index.column();
	switch ( index.column() )
	{

		case 1:
		case 2:
		{
			model->switchDataAtIndex ( index, subdirCheck->isChecked() );
			break;
		}
		default: break;
	}
}

void QRCGenerator::on_removeButton_clicked()
{
	QModelIndex selection = treeView->currentIndex();
	if ( !selection.isValid() )
	{
		//qDebug() <<"invalid selection";
		return;
	}
	if ( model->containsBaseItem ( selection ) )
	{
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "You can not remove directory from the list if it (or it's subdirectories) contain base directory. If you do not want files from this directory to be included in generated file, uncheck it." ) );
		return;
	}
	//qDebug()<<"does not contain baseItem";
	if ( model->childrenAtIndex ( selection ) > 0 )
	{
		if ( QMessageBox::question ( this, tr ( "QRCGenerator" ), tr ( "<b>Warning!</b> This will remove selected directory and all it's subdirectories from the list. If you want to remove only selected directory from the list, uncheck it.\nContinue?" ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::Yes )
		{
			model->removeAt ( selection );
		}
	}
	else
	{
		model->removeAt ( selection );
	}
	//model->getRoot()->print();
	//qDebug() <<"on_removeButton_clicked() finished!";
}

void QRCGenerator::on_langCheck_clicked ( bool sel )
{
	langCombo->setEnabled ( sel );
}
void QRCGenerator::on_aliasCheck_clicked ( bool sel )
{
	aliasEdit->setEnabled ( sel );
	helpTool->setEnabled ( sel );
	if ( sel ) aliasEdit->setFocus();
}
void QRCGenerator::on_prefixCheck_clicked ( bool sel )
{
	prefixEdit->setEnabled ( sel );
	if ( sel ) prefixEdit->setFocus();
}
void QRCGenerator::on_helpTool_clicked()
{
	QMessageBox::information ( this, tr ( "QRCGenenerator" ), tr ( "Syntax:<ul><li>$1 - filename (without extension)<li>$2 - extension<li>$3 - relative path</ul>" ) );
}

void QRCGenerator::closeEvent ( QCloseEvent *event )
{
	writeSettings();
	event->accept();
}

void QRCGenerator::readSettings()
{
	QSettings settings ( "SuslicSoft", "QRCGenerator" );
	QPoint pos = settings.value ( "pos", QPoint ( 200, 200 ) ).toPoint();
	QSize size = settings.value ( "size", QSize ( 400, 400 ) ).toSize();
	lastdir = settings.value ( "Lastdir", qApp->applicationDirPath() ).toString();

	formats = settings.value ( "Formats", QStringList() ).toStringList();
	formatsList->addItems ( formats );

	prefixEdit->addItems ( settings.value ( "Prefix", QStringList() ).toStringList() );

	aliasEdit->addItems ( settings.value ( "Alias", QStringList() <<"$1"<<"$1.$2" ).toStringList() );

	resize ( size );
	move ( pos );
}

void QRCGenerator::writeSettings()
{
	qDebug() <<"writing settings";
	QSettings settings ( "SuslicSoft", "QRCGenerator" );
	settings.setValue ( "pos", pos() );
	settings.setValue ( "size", size() );
	settings.setValue ( "Lastdir", lastdir );
	settings.setValue ( "Formats", formats );
	QStringList tmp;
	if ( prefixEdit != NULL )
	{
		for ( int i=0;i<prefixEdit->count();i++ )
		{
			tmp<<prefixEdit->itemText ( i );
		}
	}
	settings.setValue ( "Prefix", tmp );

	tmp.clear();
	if ( aliasEdit != NULL )
	{
		for ( int i=0;i<aliasEdit->count();i++ )
		{
			tmp<<aliasEdit->itemText ( i );
		}
	}
	settings.setValue ( "Alias", tmp );
}

QRCGenerator::~QRCGenerator()
{
	//qDebug() <<"~QRCGenerator begin";
	delete model;
	model = NULL;
	destroy();
}
