/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "qrcgenerator.h"

void QRCGenerator::print_lang()
{
	QFile file ( ":/lang" );
	if ( !file.open ( QFile::ReadOnly ) )
	{
		qDebug() <<"cannot open language file";
	}
	else qDebug() <<file.readAll();
}
void QRCGenerator::print_help()
{
	qDebug() <<"QRCGenerator v 0.1alpha.\n"
	<<"Options\n"
	<<"-d <base_directory>\t: base directory (the one containing *.pro file). Generated qrc file will be placed here. Files from this directory and it's subdirectories will be added to qrc file\n"
	<<"-o <filename>\t: output will be written to base_directory/filename.qrc (filename should not contain \"/\" or \"\\\" symbols!)\n"
	<<"Optional arguments:\n"
	<<"-p <prefix>\t: path prefix for all files\n"
	<<"-f <extension1;;extension2>\t: list of extensions (separated with \";;\"). Only files with this extensions will be added to output file. (default - all image formats, supported by QImageReader )\n"
	<<"-l <code>\t: iso 639 language code (see Qt documentation for details)\n\t\tyou will be asked to select language if < code> is invalid\n"
	<<"-a <alias>\t: the file tag's alias attribute\n"
	<<"\tSyntax:\n"
	<<"\t\t_1 - file name (without extension)\n"
	<<"\t\t_2 - files's extension\n"
	<<"\t\t_3 - relative path\n"
	<<"--nosubdir\t: do not add subdirectories\n"
	<<"--overwrite\t: if file <base_directory/filename.qrc> exists, overwrite it without asking.\n"
	<<"--append\t: if file <base_directory/filename.qrc> exists, append to it without asking.\n"
	<<"--help\t: print this message\n"
	<<"--help_lang\t: print this message and available language codes\n"
	<<"\nExample:\n"
	<<"\t$ ./qrcgenerator -d ./mybasedir -o application -p \"/\" -a \"_1._2\" -f \"svg;;png\" -l en --overwrite\n"
	<<"or (to select language and append to existing):\n"
	<<"\t$ ./qrcgenerator -d ./mybasedir -o application -p \"/\" -a \"_1._2\" -f \"svg;;png\" -l ? --append\n";
}
bool QRCGenerator::process_args ( QStringList args )
{
	if ( args[0].endsWith ( QFileInfo ( qApp->applicationFilePath() ).fileName() ) )	args.removeFirst();
	//qDebug() <<"args:\n"<<args;
	QString prefix;
	QString alias;
	QString lang;
	QString basedir;
	QStringList allDirs;
	bool subdirs = true;
	int overwrite = 0;
	bool have_formats = false;
	QString output_name;
	int C = args.count();

	for ( int i=0;i<C;i++ )
	{
		if ( args[i] == "-h" || args[i] == "--help" )
		{
			print_help();
			return true;
		}
		if ( args[i] == "--help_lang" )
		{
			print_help();
			print_lang();
			return true;
		}
		else if ( args[i] == "-d" && i!= ( C-1 ) )
		{
			basedir = args[i+1];
			i++;
		}
		else if ( args[i] == "-p" && i!= ( C-1 ) )
		{
			prefix = args[i+1];
			i++;
		}
		else if ( args[i] == "-f" && i!= ( C-1 ) )
		{
			formats = args[i+1].split ( ";;", QString::SkipEmptyParts );
			for ( int j=0;j<formats.count();j++ )
			{
				if ( !formats[j].startsWith ( "*." ) )	formats[j]= "*."+formats[j];
			}
			have_formats = true;
			i++;
		}
		else if ( args[i] == "-a" && i!= ( C-1 ) )
		{
			alias = args[i+1];
			i++;
		}
		else if ( args[i] == "-l" && i!= ( C-1 ) )
		{
			lang = args[i+1];
			////////////////////////
			QFile file ( ":/lang.rc" );
			if ( !file.open ( QFile::ReadOnly ) )
			{
				qDebug() <<"failed to read lang data";
				return false;
			}
			QString all = file.readAll();
			foreach ( QString chop, all.split ( "\n", QString::SkipEmptyParts ) )
			{
				langData.insert ( chop.split ( "\t" ).first(), chop.split ( "\t" ).last() );
			}
			////////////////////////
			if ( !langData.values().contains ( lang ) )
			{
				qDebug() <<"invalid language code";
				lang = QInputDialog::getItem ( this, tr ( "QRCGenerator" ), tr ( "Select language:" ), langData.keys(), 0, false );
				if ( !lang.isEmpty() ) lang = langData.value ( lang );
				qDebug() <<"lang: "<<lang;
			}
			i++;
		}
		else if ( args[i] == "-o" && i!= ( C-1 ) )
		{
			output_name = args[i+1];
			i++;
		}
		else if ( args[i] == "--nosubdir" )
		{
			subdirs = false;
		}
		else if ( args[i] == "--overwrite" )
		{
			overwrite = 1;
		}
		else if ( args[i] == "--append" )
		{
			overwrite = 2;
		}
	}
	if ( basedir.isEmpty() || output_name.isEmpty() )
	{
		qDebug() <<"not enough arguments";
		return false;
	}
	if ( !QDir ( basedir ).exists() )
	{
		qDebug() <<"invalid base directory";
		return false;
	}
	basedir = QDir ( basedir ).canonicalPath();
	qDebug() <<"canonical basedir = "<<basedir;
	if ( !have_formats )
	{
		formats = imgformats;
	}
/////////////////////
	allDirs.clear();
	if ( subdirs )
	{
		QDirIterator iter ( basedir, QDir::NoDotAndDotDot|QDir::Dirs, QDirIterator::Subdirectories );
		while ( iter.hasNext() ) allDirs<<iter.next();
	}
	allDirs.insert ( 0, basedir );
//////////////////////
	QString outstr = gen_output ( basedir, &allDirs, prefix, alias, lang, "_" );
//////////////////////
	output_name = basedir+"/"+output_name;
	if ( !output_name.endsWith ( ".qrc", Qt::CaseInsensitive ) ) output_name+=".qrc";
	qDebug() <<"output_name = "<<output_name;
	return saveCore ( output_name , &outstr, overwrite );
}
