/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "qrcgenerator.h"

void QRCGenerator::on_generateButton_clicked()
{
	if ( model->getRoot()->fchild == NULL )
	{
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "Directories list is empty!" ) );
		return;
	}
	if ( model->getBase() == NULL )
	{
		if ( QMessageBox::question ( this, tr ( "QRCGenerator" ), tr ( "You must select one of the directories to be the \"base\" one (base directory should be the one containing *.pro file ).\nSelect the first one to be base directory?" ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::Yes )
		{
			if ( !model->setFirstAsBase() )
			{
				qDebug() <<"DirMode: setFirstAsBase failed";
				return;
			}
		}
		else return;
	}
	baseDir = model->getBase()->dataAt ( 0 ).toString();
	//qDebug() <<"QRCGenerator::on_generateButton_clicked(): baseDir = "<<baseDir;

	QStringList allDirs = model->getRoot()->getDirData ( false );
	if ( allDirs.isEmpty() )
	{
		//qDebug() <<"BUG: getDirData() returned empty list!";
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "Please select at least one directory!" ) );
		return;
	}

	QString prefix;
	if ( prefixCheck->isChecked() )
	{
		prefix = prefixEdit->currentText();
	}

	QString alias;
	if ( aliasCheck->isChecked() )
	{
		alias = aliasEdit->currentText();
	}

	if ( !plainTextEdit->toPlainText().isEmpty() )
	{
		if ( QMessageBox::question ( this, tr ( "QRCGenerator" ), tr ( "Clear text edit before generation?\n(If you select \"No\", generated text will be appended to existing)" ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::Yes )
		{
			plainTextEdit->setPlainText ( gen_output ( baseDir, &allDirs, prefix , alias , langCheck->isChecked() ? langData.value ( langCombo->currentText() ) : "", "$" ) );
		}
		else
		{
			QString text = plainTextEdit->toPlainText() + "\n" + gen_output ( baseDir, &allDirs, prefix , alias, langCheck->isChecked() ? langData.value ( langCombo->currentText() ) : "", "$" );
			QRegExp combine_rx ( "\\<(\\s*)\\/RCC(\\s*)\\>(\\s*)(\\n*)(\\s*)\\<(\\s*)\\!DOCTYPE(\\s*)RCC(\\s*)\\>(\\s*)\\<(\\s*)RCC(\\s*)version(\\s*)=(\\s*)\"1.0\"(\\s*)\\>(\\s*)(\\n*)(\\s*)", Qt::CaseInsensitive );
			text.replace ( combine_rx, "" );
			plainTextEdit->setPlainText ( text );
		}
	}
	else plainTextEdit->setPlainText ( gen_output ( baseDir, &allDirs, prefix , alias, langCheck->isChecked() ? langData.value ( langCombo->currentText() ) : "", "$" ) );
}

QString QRCGenerator::gen_output ( QString baseD, QStringList* allDirs, QString prefix, QString alias, QString lang, QString dollar )
{
	QString OUTPUT;
	//qDebug() <<"prefix= "<<prefix;
	//qDebug() <<"alias= "<<alias;
	//qDebug() <<"lang= "<<lang;
	QStringList img_aliases;
	OUTPUT="<!DOCTYPE RCC><RCC version=\"1.0\">\n<qresource";
	if ( !prefix.isEmpty() ) OUTPUT+= ( " prefix=\""+prefix+"\"" );
	if ( !lang.isEmpty() ) OUTPUT+= ( " lang=\""+lang+"\"" );
	OUTPUT+=">\n";
	QDir basedir ( baseD );
	for ( int i=0;i<allDirs->count();i++ )
	{
		QString base = basedir.relativeFilePath ( allDirs->at ( i ) );
		while ( base.startsWith ( "/" ) ) base.remove ( 0,1 );
		if ( !base.isEmpty() && !base.endsWith ( "/" ) ) base+="/";
		qDebug() <<"base for"<<allDirs->at ( i ) <<" = "<<base;
		QStringList img_files = QDir ( allDirs->at ( i ) ).entryList ( formats, QDir::Files, QDir::Name );
		for ( int j=0;j<img_files.count();j++ )
		{
			QString outstr = "\t<file ";
			QString imgfile = base+img_files[j];
			if ( !alias.isEmpty() )
			{
				QFileInfo fi ( img_files[j] );
				QString imgalias = alias;
				imgalias.replace ( dollar+"1", fi.baseName() ).replace ( dollar+"2", fi.suffix() ).replace ( dollar+"3", imgfile );
				if ( img_aliases.contains ( imgalias ) )
				{
					imgalias = QInputDialog::getText ( this, tr ( "QRCGenerator" ), tr ( "This qrc file has already file tag with alias %1\nPlease, specify another one:" ).arg ( imgalias ), QLineEdit::Normal, imgalias );
				}
				outstr+= ( "alias=\""+imgalias+"\"" );
			}
			outstr+= ( ">"+imgfile+"</file>\n" );
			OUTPUT+=outstr;
		}
	}
	OUTPUT+="</qresource>\n</RCC>\n";
	//qDebug()<<OUTPUT;
	return OUTPUT;
}
