/***************************************************************************
 *   Copyright (C) 2009 by Koptsow Dmitriy   *
 *   suslic-2012@rambler.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "qrcgenerator.h"

void QRCGenerator::on_saveButton_clicked()
{
	if ( baseDir.isEmpty() )
	{
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "You do not have base directory set! Please, press \"Generate\" button first" ) );
		return;
	}
	if ( plainTextEdit->toPlainText().isEmpty() )
	{
		//qDebug()<<"nothing to save";
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "Text edit is empty!" ) );
		return;
	}
	QString file = QInputDialog::getText ( this, tr ( "QRCGenerator" ), tr ( "File name\n(will be located in %1):\n" ).arg ( baseDir ) );
	if ( file.isEmpty() ) return;
	if ( !file.endsWith ( ".qrc", Qt::CaseInsensitive ) ) file+=".qrc";
	file = baseDir+"/"+file;
	QString text = plainTextEdit->toPlainText();
	saveCore ( file, &text );
}

bool QRCGenerator::saveCore ( QString file, QString* data, int overwrite )
{
	QFile ofile ( file );
	QString appendstr;
	bool append = false;
	if ( overwrite == 2 ) append = true;
	if ( ofile.exists() && overwrite == 0 )
	{
		switch ( question ( file ) )
		{
			case 0: break;
			case 1: append = true;break;
			default: return false; break;
		}
	}
	if ( append && ofile.exists() )
	{
		if ( !ofile.open ( QFile::ReadOnly|QFile::Text ) )
		{
			QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "Cannot read file %1" ).arg ( file ) );
			return false;
		}
		appendstr = ofile.readAll();
		appendstr+= ( *data );
		QRegExp combine_rx ( "\\<(\\s*)\\/RCC(\\s*)\\>(\\s*)(\\n*)(\\s*)\\<(\\s*)\\!DOCTYPE(\\s*)RCC(\\s*)\\>(\\s*)\\<(\\s*)RCC(\\s*)version(\\s*)=(\\s*)\"1.0\"(\\s*)\\>(\\s*)(\\n*)(\\s*)", Qt::CaseInsensitive );
		appendstr.replace ( combine_rx, "" );
		ofile.close();
	}
	else	appendstr = ( *data );
	if ( !ofile.open ( QFile::WriteOnly|QFile::Text ) )
	{
		QMessageBox::warning ( this, tr ( "QRCGenerator" ), tr ( "Cannot write to file %1" ).arg ( file ) );
		return false;
	}
	//qDebug() <<"ofile opened in overwrite mode";

	QTextStream out ( &ofile );
	out<< appendstr;
	ofile.close();
	return true;
}

int QRCGenerator::question ( QString fname )
{
	QMessageBox msgBox;
	msgBox.setText ( tr ( "Attention! File %1 exists!" ).arg ( fname ) );
	msgBox.setInformativeText ( tr ( "You can <b>overwrite</b> existing file with new one, or you can <b>append</b> new data to existing one.<br>That means, generated data will be added to the end of existing file (useful when \"some resources, such as translation files and icons, need to change based on the user's locale\" (from Qt manual) ).<br>It is assumed, that exiting file is valid, i.e. ends with \"&lt;/RCC&gt;\" (case insensitive).<br><b>Warning!</b> QRCGenerator does not parse existing file, so it won't warn you about repeating aliases etc." ) );
	QPushButton *overButton = new QPushButton ( tr ( "&Overwrite" ) );
	msgBox.addButton ( overButton, QMessageBox::ActionRole );
	QPushButton *mergeButton = new QPushButton ( tr ( "&Append" ) );
	msgBox.addButton ( mergeButton,  QMessageBox::ActionRole );
	QPushButton *cancelButton = msgBox.addButton ( QMessageBox::Cancel );
	msgBox.setDefaultButton ( cancelButton );
	msgBox.exec();

	if ( msgBox.clickedButton() == overButton ) return 0;
	else if ( msgBox.clickedButton() == mergeButton ) return 1;
	else if ( msgBox.clickedButton() == cancelButton ) return 2;
	return 2;
}
